/*
 * Control AMD SB7XX gameport
 */

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <pciaccess.h>


void set_gameport_config_reg(struct pci_device * dev, int state) {
  uint8_t reg;
  int err;

  if ((err = pci_device_cfg_read_u8(dev, &reg, 0x47))) {
    printf("config read error: %s\n", strerror(err));
  }

  if ((err = pci_device_cfg_write_u8(dev, state ?  (reg | 0x10) : (reg & ~0x10), 0x47))) {
    printf("config write error: %s\n", strerror(err));
  }
}


void print_gameport_status(struct pci_device * dev) {
  uint8_t reg;
  int err;
  if ((err = pci_device_cfg_read_u8(dev, &reg, 0x47))) {
    printf("config register read error: %s\n", strerror(err));
  }
  
  printf("gameport status: %s\n", (reg >> 4 & 0x01) ? "enabled" : "disabled");
}


struct pci_device * find_lpc_bridge() {
  struct pci_device_iterator * iter;
  struct pci_device * dev;

  iter = pci_id_match_iterator_create(NULL);  
  while ((dev = pci_device_next(iter))) {
    pci_device_probe(dev);
    if (dev->vendor_id == 0x1002 && dev->device_id == 0x439d) {
      pci_iterator_destroy(iter);
      return dev;
    }
  }
  
  pci_iterator_destroy(iter);
  return NULL;
}

void print_help() {
  printf("usage: sb7xxgameport [status|on|off]\n");
}

int main (int argc, char **argv) {
  int err;
  struct pci_device * dev;
  
  if (getuid() != 0) {
    printf("sb7xxgameport requires root privileges.\n");
    return -1;
  }

  if ((err = pci_system_init())) {
    printf("pci init error: %s \n", strerror(err));
    return -1;
  }

  if (!(dev = find_lpc_bridge())) {
    printf("Sb7xx lpc bridge not found\n");
    return -1;
  }
  
  if (argc > 1) {
    if (!strcmp(argv[1], "status")) {
      print_gameport_status(dev);
    } else if (!strcmp(argv[1], "on")) {
      set_gameport_config_reg(dev, 1);
    } else if (!strcmp(argv[1], "off")) {
      set_gameport_config_reg(dev, 0);
    } else {
      print_help();
    }
  } else {
    print_help();
  }

  return 0;
}
